#!/bin/ruby

begin; require 'rubygems'; rescue; end

require 'gosu'

# Screen size
W = 640
H = 480

# Full-screen mode
FS = false

# Update frequency (Hz)
FPS = 60

# The double-click time-out in milliseconds
# (two clicks separated by more time than this are not a double-click)
DBLCLICKTIME = 500

# The maximum distance between the clicks in a double-click
DBLCLICKDIST = 5

class Thing

  attr_accessor :x, :y

  def initialize(win, x, y)
    @win = win

    @pointer = Gosu::Image.load_tiles(win, 'thing.png', -2, -1, false)

    @clicked = false
    @here_x = 0
    @here_y = 0
    @now = 0
    @state = 0

    @x = x
    @y = y
  end

  def button_up(id)
    if id == Gosu::Button::MsLeft then
      if @clicked then
        # Check the time between the two clicks
        fast_enough = (Gosu::milliseconds - @now < DBLCLICKTIME)

        # Check how much the mouse moved between the two clicks
        close_enough = (Gosu::distance(@here_x, @here_y, @win.mouse_x, @win.mouse_y) <= DBLCLICKDIST)

        # If they're both within range, there was a double-click
        double_click if fast_enough and close_enough
        @clicked = false
      else
        @now = Gosu::milliseconds
        @here_x = @win.mouse_x
        @here_y = @win.mouse_y
        @clicked = true
      end

    end
  end

  def draw
    @pointer[@state].draw_rot(@x, @y, 0, 0)
  end

  def double_click
    @state = (@state + 1) % 2
    puts 'Double-click detected!'
  end

end # Thing class


### MAIN WINDOW CLASS ###
class GfxWindow < Gosu::Window

  def initialize
    # Initialize the base class
    super W, H, FS, 1.0 / FPS * 1000
    self.caption = "Gosu Template"

    @info = Gosu::Font.new(self, Gosu::default_font_name, 14)
    @pointer = Gosu::Image.new(self, 'pointer.png', false)

    @thing = Thing.new(self, W / 2, H / 2)

    @last_time = Gosu::milliseconds
  end

  def button_up(id)
    if id == Gosu::Button::KbEscape then
      close
    end

    @thing.button_up(id)
  end

  def draw
    @info.draw("x,y: #{self.mouse_x}, #{self.mouse_y} ms: #{Gosu::milliseconds() - @last_time}", 10, 10, 0)

    @pointer.draw(@x, @y, 0)
    @thing.draw

    @last_time = Gosu::milliseconds
  end

  def update
    @x = self.mouse_x
    @y = self.mouse_y
  end

  def needs_cursor?()
    false
  end
end

begin
  GfxWindow.new.show
rescue Exception => e
  puts e.message, e.backtrace
  gets
end
